/**
 * App component script
 */
import { Component, ViewEncapsulation } from '@angular/core';
import { AssignmentModel, AsyncHelper, DataGenerator, DateHelper, DurationField, EventModel, EventStore, ProjectModel, ResourceModel, Scheduler, WidgetHelper } from '@bryntum/scheduler';
import { schedulerConfig } from './app.config';
import { dayOffEventModel } from './dayOffEventModel';
import { eventBodyTemplate } from './EventBodyTemplate';
import { eventRenderer } from './EventRender';
import { schedulerEventModel } from './schedulerEventModel';
import { shiftEventModel } from './shiftEventModel';

@Component({
    selector      : 'app-root',
    templateUrl   : './app.component.html',
    styleUrls     : ['./app.component.scss'],
    encapsulation : ViewEncapsulation.None
})

export class AppComponent  {
    public scheduler: Scheduler;
    public resources = [];
    public events:schedulerEventModel[] = [];
    public assignments = [];
    //public schedulerConfig = schedulerConfig;

public filterEvents(){
    if (!this.scheduler.isDestroyed)
    {
        this.scheduler.destroy();
    }
    this.createScheduler();
    this.events = [...this.events.filter(x=> x.id>1)];
    //recargo los assignments como en proyecto final
    this.assignments = [];
    this.events.forEach(event =>
        this.assignments.push({
            id       : 'a' + event.id,
            event    : event.id,
            resource : event.resourceId
        })
        );
        this.refreshData(this.scheduler.endDate);

}
    public assignEvents(){

        this.createScheduler();
        this.generateResources();
    }

    private createScheduler() {
        this.scheduler = new Scheduler({
            appendTo: 'container',
            eventStyle: 'border',
            rowHeight: 50,
            useInitialAnimation: false,
            enableEventAnimations: false,
            //this.generateResources,
            columns: [
                { type: 'rownumber' },
                { text: 'Id', field: 'id', width: 50, hidden: true },
                { text: 'First name', field: 'name', flex: 1 },
                // { text : 'Surname', field : 'surName', flex : 1 },
                // { text : 'Score', field : 'score', type : 'number', flex : 1 }
            ],
            features: {
                // Features that can be toggled in Custom mode
                dependencies: {
                    disabled: true
                },
                resourceTimeRanges: {
                    disabled: true
                },
                timeRanges: {
                    disabled: true,
                    enableResizing: true
                },
                eventNonWorkingTime: {
                    disabled: true
                },
                nonWorkingTime: {
                    disabled: true
                },
                // Turn off schedule tooltip to boost scrolling performance a bit
                scheduleTooltip: true
            },
            eventBodyTemplate: eventBodyTemplate,
            eventRenderer: eventRenderer
        });
    }

    async generateResources() {
        const
            today                 = DateHelper.clearTime(new Date()),
            colors                = ['cyan', 'green', 'indigo'];


        let schedulerEndDate = today,
            i: number;

        console.time('generate');



        const res1 =  new ResourceModel({
            id       : 1,
            name     : 'Alfonso'
        });
        const res2 = new ResourceModel({
            id         : 2,
            name       : 'Pepe'
        });
        const res3 = new ResourceModel({
            id         : 3,
            name       : 'Manue'
        });


        this.resources.push(res1);
        this.resources.push(res2);
        this.resources.push(res3);

        for (i = 0; i < 3; i++){
            const
                startDate = DateHelper.add(today, Math.round(Math.random() * (i + 1) * 20), 'days'),
                duration  = Math.round(Math.random() * 9) + 2,
                endDate   = DateHelper.add(startDate, duration, 'days'),
                eventId   = i+1;

                if(i===0){
                    const ev:shiftEventModel = new shiftEventModel (
                        );

                                ev.id         = eventId;
                                ev.name       = 'Task # Finalshift';
                                ev.startDate = startDate;
                                ev.duration = duration;
                                ev.endDate = endDate;
                                ev.eventColor = colors[this.resources.length % 3];
                                ev.shiftId = eventId;
                                this.events.push(ev);
                }
                if(i===1 || i===2){
                    const ev:dayOffEventModel = new dayOffEventModel (
                        );

                                ev.id         = eventId;
                                ev.name       = 'Task #dayOff';
                                ev.startDate = startDate;
                                ev.duration = duration;
                                ev.endDate = endDate;
                                ev.eventColor = colors[this.resources.length % 3];
                                ev.dayOffType = "paternidad";
                                this.events.push(ev);
                }

            this.assignments.push({
                id       : 'a' + eventId,
                event    : eventId,
                resource : this.resources[i].id
            });


            if (endDate > schedulerEndDate) {
                schedulerEndDate = endDate;
            }
        }

        //}

        console.timeEnd('generate');

        //this.scheduler.mask.text = 'Loading data';
        this.refreshData(schedulerEndDate);

    }
    async refreshData(schedulerEndDate: Date) {
        // Give the UI a chance to catch up, to update the mask before proceeding
        await AsyncHelper.sleep(100);

        console.time('data');

        this.scheduler.suspendRefresh();
        this.scheduler.endDate = schedulerEndDate;
        this.scheduler.project = new ProjectModel({
            assignmentStore : {
                data       : this.assignments
            },
            resourceStore : {
                useRawData : true,
                data       : this.resources
            },
            eventStore : {
                useRawData : true,
                data       : this.events
            }
        });
        this.scheduler.resumeRefresh(true);

        await this.scheduler.project.commitAsync();

        console.timeEnd('data');

       //mask.close();
   }

}


